@@ -4,7 +4,7 @@ library(jit)
 
 GCTORTURE.FLAG <- 0
 TRACE.FLAG <- 0
-QUICK.FLAG <- TRUE    # FALSE for full test, TRUE for quick test
+QUICK.FLAG <- FALSE   # FALSE for full test, TRUE for quick test
 NREPEATS <- 5         # for calculating time stddev
 if (QUICK.FLAG == 1)
     NREPEATS <- 1
