@@ -41,6 +41,9 @@
 #include <windows.h>		/* for CreateEvent,.. */
 #include <process.h>		/* for _beginthread,... */
 #include <io.h>			/* for isatty, chdir */
+#ifdef _MSC_VER  /* for chdir */
+# include <direct.h>
+#endif
 #include "run.h"
 #include "Startup.h"
 #include <stdlib.h>		/* for exit */
@@ -60,7 +63,7 @@ Rboolean DebugMenuitem = FALSE;  /* exported for rui.c */
 static FILE *ifp = NULL;
 static char ifile[MAX_PATH] = "\0";
 
-__declspec(dllexport) UImode  CharacterMode;
+__declspec(dllexport) UImode  CharacterMode = RGui; /* some compilers want initialized for export */
 int ConsoleAcceptCmd;
 void set_workspace_name(const char *fn); /* ../main/startup.c */
 
@@ -506,7 +509,6 @@ void R_CleanUp(SA_TYPE saveact, int status, int runLast)
      */
 
 
-/* As from R 2.7.0 we assume file[i] and pager are in UTF-8 */
 extern FILE *R_wfopen(const wchar_t *filename, const wchar_t *mode);
 extern size_t Rf_utf8towcs(wchar_t *wc, const char *s, size_t n);
 
@@ -515,7 +517,6 @@ int R_ShowFiles(int nfile, const char **file, const char **headers,
 {
     int   i;
     char  buf[1024];
-    wchar_t wfn[PATH_MAX+1];
 
     if (nfile > 0) {
 	if (pager == NULL || strlen(pager) == 0)
@@ -523,19 +524,18 @@ int R_ShowFiles(int nfile, const char **file, const char **headers,
 	for (i = 0; i < nfile; i++) {
 	    if(!access(file[i], R_OK)) {
 		if (!strcmp(pager, "internal")) {
-		    newpager(wtitle, file[i], CE_UTF8, headers[i], del);
+		    newpager(wtitle, file[i], CE_NATIVE, headers[i], del);
 		} else if (!strcmp(pager, "console")) {
 		    size_t len;
 		    FILE *f;
-		    Rf_utf8towcs(wfn, file[i], PATH_MAX+1);
-		    f = R_wfopen(wfn, L"rt");
+		    f = R_fopen(file[i], "rt");
 		    if(f) {
 			while((len = fread(buf, 1, 1023, f))) {
 			    buf[len] = '\0';
 			    R_WriteConsole(buf, strlen(buf));
 			}
 			fclose(f);
-			if (del) DeleteFileW(wfn);
+			if (del) DeleteFile(file[i]);
 		    }
 		    else {
 			snprintf(buf, 1024,
@@ -604,6 +604,7 @@ int R_EditFiles(int nfile, const char **file, const char **title,
     return 1;
 }
 
+#if 0
 /* Prompt the user for a file name.  Return the length of */
 /* the name typed.  On Gui platforms, this should bring up */
 /* a dialog box so a user can choose files that way. */
@@ -614,6 +615,7 @@ int R_ChooseFile(int new, char *buf, int len)
 {
     return DialogSelectFile(buf, len);
 }
+#endif
 
 /* code for R_ShowMessage, R_YesNoCancel */
 
@@ -780,7 +782,7 @@ char *PrintUsage(void)
 	msg2b[] =
 	"  --max-mem-size=N      Set limit for memory to be used by R\n  --max-ppsize=N        Set max size of protect stack to N\n",
 	msg3[] =
-	"  -q, --quiet           Don't print startup message\n  --silent              Same as --quiet\n  --slave               Make R run as quietly as possible\n  --verbose             Print more information about progress\n  --args                Skip the rest of the command line\n",
+	"  -q, --quiet           Don't print startup message\n  --silent              Same as --quiet\n  --slave               Make R run as quietly as possible\n  --verbose             Print more information about progress\n  --internet2           Use Internet Explorer for proxies etc.\n  --args                Skip the rest of the command line\n",
 	msg4[] =
 	"  --ess                 Don't use getline for command-line editing\n                          and assert interactive use\n  -f file               Take input from 'file'\n  --file=file           ditto\n  -e expression         Use 'expression' as input\n\nOne or more -e options can be used, but not together with -f or --file\n",
 	msg5[] = "\nAn argument ending in .RData (in any case) is taken as the path\nto the workspace to be restored (and implies --restore)";
