@@ -81,6 +81,7 @@ SEXP do_writeClipboard(SEXP, SEXP, SEXP, SEXP);
 SEXP do_winprogressbar(SEXP, SEXP, SEXP, SEXP);
 SEXP do_closewinprogressbar(SEXP, SEXP, SEXP, SEXP);
 SEXP do_setwinprogressbar(SEXP, SEXP, SEXP, SEXP);
+SEXP do_setInternet2(SEXP, SEXP, SEXP, SEXP);
 #endif
 
 SEXP do_abbrev(SEXP, SEXP, SEXP, SEXP);
@@ -264,7 +265,6 @@ SEXP do_image(SEXP, SEXP, SEXP, SEXP);
 SEXP do_indexsearch(SEXP, SEXP, SEXP, SEXP);
 SEXP do_inherits(SEXP, SEXP, SEXP, SEXP);
 SEXP do_inspect(SEXP, SEXP, SEXP, SEXP);
-SEXP do_int_unzip(SEXP, SEXP, SEXP, SEXP);
 SEXP do_intToUtf8(SEXP, SEXP, SEXP, SEXP);
 SEXP do_interactive(SEXP, SEXP, SEXP, SEXP);
 SEXP do_internal(SEXP, SEXP, SEXP, SEXP);
