@@ -479,7 +479,6 @@ void genjitSubset(CSEXP x, CSEXP subset);
 void genjitSubset2(CSEXP x, CSEXP subset);
 void genjitUnary(JIT_OPCODE opcode, CSEXP x);
 void initJit(void);
-void jitContextChangedAux(int nest);
 void jitOff(Rboolean issueWarning);
 void jitPreventLocalRebindingAux(CSEXP sym, CSEXP val, CSEXP env);
 void jitSuspendAux(const char msg[]);
@@ -492,6 +491,7 @@ void pushJitState(CSEXP call, unsigned newState);
 void traceBinding(CSEXP loc, CSEXP val);
 void traceSymBinding(CSEXP sym, CSEXP val);
 #endif
+void saveJitStackPointer(Rboolean save);
 
 // these inline functions are for efficiency without clutter
 
@@ -508,11 +508,6 @@ static R_INLINE void genjitBin(JIT_OPCODE opcode, CSEXP x,
         UNPROTECT(1);
     }
 }
-static R_INLINE void jitContextChanged(int nest) // called by findcontext()
-{
-    if (jitState & ~JITS_IDLE)
-        jitContextChangedAux(nest);
-}
 static R_INLINE void jitPreventLocalRebinding(CSEXP sym, CSEXP val, CSEXP env)
 {
     if (jitState & JITS_COMPILING_STATES)
