@@ -17,6 +17,7 @@
 ####------ str : show STRucture of an R object
 str <- function(object, ...) UseMethod("str")
 
+## FIXME: convert to use sQuote
 str.data.frame <- function(object, ...)
 {
     ## Method to 'str' for  'data.frame' objects
@@ -44,7 +45,7 @@ str.POSIXt <- function(object, ...) {
     cl <- oldClass(object)
     ## be careful to be fast for large object:
     n <- length(object)
-    if(n >= 1000) object <- object[1:1000]
+    if(n >= 1000) object <- object[1L:1000]
 
     give.length <- TRUE ## default
     ## use 'give.length' when specified, else default = give.head
@@ -94,7 +95,7 @@ str.default <-
     ## Author: Martin Maechler <maechler@stat.math.ethz.ch>	1990--1997
 
     ## Get defaults for these
-    oDefs <- c("vec.len", "digits.d", "strict.width")
+    oDefs <- c("vec.len", "digits.d", "strict.width", "formatNum")
     ## from
     strO <- getOption("str")
     if (!is.list(strO)) {
@@ -185,7 +186,7 @@ str.default <-
 	    if(is.d.f) std.attr <- c(std.attr, "class", "row.names")
 	    else cat(" ", if(i.pl)"pair", "list()\n",sep="")
 	} else { # list, length >= 1 :
-	    if(irregCl <- has.class && identical(object[[1]], object)) {
+	    if(irregCl <- has.class && identical(object[[1L]], object)) {
 		le <- length(object <- unclass(object))
 		std.attr <- c(std.attr, "class")
 	    }
@@ -207,7 +208,7 @@ str.default <-
 		    else { max.ncnam <- max(nchar(nam.ob, type="w"))
 			   format(nam.ob, width = max.ncnam, justify="left")
 		       }
-		for(i in 1:le) {
+		for(i in 1L:le) {
 		    cat(indent.str, comp.str, nam.ob[i], ":", sep="")
 		    envir <- # pass envir for 'promise' components:
 			if(typeof(object[[i]]) == "promise") {
@@ -233,15 +234,16 @@ str.default <-
 		mod <- substr(mode(object), 1, 4)
 		if     (mod == "nume")
 		    mod <- if(is.integer(object)) "int"
-		    else if(has.class) cl[1] else "num"
+		    else if(has.class) cl[1L] else "num"
 		else if(mod == "char") { mod <- "chr"; char.like <- TRUE }
 		else if(mod == "comp") mod <- "cplx" #- else: keep 'logi'
 		if(is.array(object)) {
-		    di. <- dim(object)
+		    rnk <- length(di. <- dim(object))
 		    di <- P0(ifelse(di. > 1, "1:",""), di.,
 			     ifelse(di. > 0, "" ," "))
-		    le.str <- paste(c("[", P0(di[-length(di)], ", "),
-				      di[length(di)], "]"), collapse = "")
+		    pDi <- function(...) paste(c("[", ..., "]"), collapse = "")
+		    le.str <- (if(rnk == 1) pDi(di[1L], "(1d)") else
+			       pDi(P0(di[-rnk], ", "), di[rnk]))
 		    std.attr <- "dim" #- "names"
 		} else if(!is.null(names(object))) {
 		    mod <- paste("Named", mod)
@@ -290,8 +292,8 @@ str.default <-
 #	    std.attr <- c(std.attr, "tspar")
 	} else if(stats::is.ts(object)) {
 	    tsp.a <- stats::tsp(object)
-	    str1 <- P0(" Time-Series ", le.str, " from ", format(tsp.a[1]),
-		       " to ", format(tsp.a[2]), ":")
+	    str1 <- P0(" Time-Series ", le.str, " from ", format(tsp.a[1L]),
+		       " to ", format(tsp.a[2L]), ":")
 	    std.attr <- c("tsp","class") #- "names"
 	} else if (is.factor(object)) {
 	    nl <- length(lev.att <- levels(object))
@@ -305,8 +307,8 @@ str.default <-
 		## as from 2.1.0, quotes are included ==> '-2':
 		lenl <- cumsum(3 + (nchar(lev.att, type="w") - 2))# level space
 		ml <- if(nl <= 1 || lenl[nl] <= 13)
-		    nl else which(lenl > 13)[1]
-		lev.att <- maybe_truncate(lev.att[1:ml])
+		    nl else which(lenl > 13)[1L]
+		lev.att <- maybe_truncate(lev.att[1L:ml])
 ##		if((d <- lenl[ml] - if(ml>1)18 else 14) >= 3)# truncate last
 ##		    lev.att[ml] <-
 ##			P0(substring(lev.att[ml],1, nchar(lev.att[ml])-d), "..")
@@ -339,7 +341,16 @@ str.default <-
 	    cat("Class", if(length(cl) > 1) "es",
 		" '", paste(cl, collapse = "', '"), "' ", sep="")
 	    ## If there's a str.<method>, it should have been called before!
-	    str(unclass(object),
+	    uo <- unclass(object)
+	    if(!is.null(attributes(uo)$class)) {
+		## another irregular case
+		xtr <- c(if(identical(uo, object)) { # trap infinite loop
+		    class(uo) <- NULL
+		    "unclass()-immune"
+		} else if(!is.object(object)) "not-object")
+		if(!is.null(xtr)) cat("{",xtr,"} ", sep="")
+	    }
+	    str(uo,
 		max.level = max.level, vec.len = vec.len, digits.d = digits.d,
 		indent.str = paste(indent.str,".."), nest.lev = nest.lev + 1,
 		nchar.max = nchar.max, give.attr = give.attr, width=width)
@@ -384,10 +395,10 @@ str.default <-
 		le <- length(object) # is > 1 e.g. for {A;B} language
 		format.fun <- function(x)x
 		v.len <- round(.5 * v.len)
-		if(le > 1 && typ=="language" && object[1] == "{" && object[le]=="}") {
+		if(le > 1 && typ=="language" && object[1L] == "{" && object[le]=="}") {
 		    v.len <- v.len + 2
 		    if(le >= 3) {
-			object <- c(object[1],
+			object <- c(object[1L],
 				    paste(sub("^ +", " ", object[2:(le-1)]),
 					  collapse = ";"),
 				    object[le])
@@ -422,7 +433,7 @@ str.default <-
 		le <- length(object <- as.character(object))
 	    if(int.surv || (all(ao > 1e-10 | ao==0) && all(ao < 1e10| ao==0) &&
 			    all(abs(ob - signif(ob, digits.d)) <= 9e-16*ao))) {
-		if(!iSurv || di.[2] == 2) # "Surv" : implemented as matrix
+		if(!iSurv || di.[2L] == 2) # "Surv" : implemented as matrix
 		    ## use integer-like length
 		    v.len <- iv.len
 		format.fun <- formatNum
@@ -441,7 +452,7 @@ str.default <-
 	if(char.like) {
 	    ## if object is very long, drop the rest which won't be used anyway:
 	    max.len <- max(100, width %/% 3 + 1, if(!missing(vec.len)) vec.len)
-	    if(le > max.len) object <- object[1:max.len]
+	    if(le > max.len) object <- object[1L:max.len]
 	    encObj <- encodeString(object, quote= '"', na.encode= FALSE)
 					#O: encodeString(object)
 	    v.len <-
@@ -452,26 +463,26 @@ str.default <-
 		else round(v.len)
 	    ile <- min(le, v.len)
 	    if(ile >= 1) ## truncate if LONG char:
-		object <- maybe_truncate(encObj[1:ile])
+		object <- maybe_truncate(encObj[1L:ile])
 					#O: encodeString(object, quote= '"', na.encode= FALSE)
 	    formObj <- function(x) paste(as.character(x), collapse=" ")
 	}
 	else {
 	    if(!exists("format.fun", inherits=TRUE)) #-- define one --
 		format.fun <-
-		    if(mod == 'num' || mod == 'cplx') format else as.character
+		    if(mod == "num" || mod == "cplx") format else as.character
 	    ## v.len <- max(1,round(v.len))
 	    ile <- min(v.len, le)
 	    formObj <- function(x) paste(format.fun(x), collapse = " ")
 	}
 
 	cat(if(give.head) P0(str1, " "),
-	    formObj(if(ile >= 1) object[1:ile] else if(v.len > 0) object),
+	    formObj(if(ile >= 1) object[1L:ile] else if(v.len > 0) object),
 	    if(le > v.len) " ...", "\n", sep="")
 
     } ## else (not function nor list)----------------------------------------
 
-    if(give.attr) { ## possible: || has.class && any(cl == 'terms')
+    if(give.attr) { ## possible: || has.class && any(cl == "terms")
 	nam <- names(a)
 	for (i in seq_along(a))
 	    if (all(nam[i] != std.attr)) {# only `non-standard' attributes:
@@ -512,8 +523,24 @@ print.ls_str <- function(x, max.level = 1, give.attr = FALSE, ...)
     M <- attr(x, "mode")
     for(nam in x) {
 	cat(nam, ": ")
-	str(get(nam, envir = E, mode = M),
-	    max.level = max.level, give.attr = give.attr, ...)
+	## check missingness, e.g. inside debug(.) :
+
+##__ Why does this give	 too many <missing> in some case?
+##__	if(eval(substitute(missing(.), list(. = as.name(nam))),
+##__		envir = E))
+##__	    cat("<missing>\n")
+##__	else
+##__	    str(get(nam, envir = E, mode = M),
+##__		max.level = max.level, give.attr = give.attr, ...)
+
+	o <- tryCatch(get(nam, envir = E, mode = M), error = function(e)e)
+	if(inherits(o, "error")) {
+	    if(length(grep("missing|not found", o$message)))
+		cat("<missing>\n")
+	    else stop(o$message)
+	}
+	else
+	    str(o, max.level = max.level, give.attr = give.attr, ...)
     }
     invisible(x)
 }
