@@ -189,18 +189,14 @@ static R_INLINE double R_log(double x) {
     return x > 0 ? log(x) : x < 0 ? R_NaN : R_NegInf;
 }
 
-#ifdef POW_DIRTY
-
-# define R_pow	pow
-
-#else
 double R_pow(double x, double y) /* = x ^ y */
 {
     if(x == 1. || y == 0.)
 	return(1.);
     if(x == 0.) {
 	if(y > 0.) return(0.);
-	/* y < 0 */ return(R_PosInf);
+	else if(y < 0) return(R_PosInf);
+	else return(y); /* NA or NaN, we assert */
     }
     if (R_FINITE(x) && R_FINITE(y))
 /* work around a bug in May 2007 snapshots of gcc pre-4.3.0, also
@@ -235,7 +231,6 @@ double R_pow(double x, double y) /* = x ^ y */
     return(R_NaN);		/* all other cases: (-Inf)^{+-Inf,
 				   non-int}; (neg)^{+-Inf} */
 }
-#endif
 
 double R_pow_di(double x, int n)
 {
@@ -627,7 +622,6 @@ static SEXP integer_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2, SEXP lcall)
     else
 	ans = allocVector(INTSXP, n);
     if (n1 == 0 || n2 == 0) return(ans);
-    PROTECT(ans);
 
     /* note: code below was surely wrong in DIVOP and POWOP cases,
        since ans was a REALSXP.
@@ -768,10 +762,8 @@ static SEXP integer_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2, SEXP lcall)
 
 
     /* quick return if there are no attributes */
-    if (ATTRIB(s1) == R_NilValue && ATTRIB(s2) == R_NilValue) {
-        UNPROTECT(1);
+    if (ATTRIB(s1) == R_NilValue && ATTRIB(s2) == R_NilValue)
 	return ans;
-    }
 
     /* Copy attributes from longer argument. */
 
@@ -784,7 +776,6 @@ static SEXP integer_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2, SEXP lcall)
     else
 	copyMostAttrib(s2, ans);
 
-    UNPROTECT(1);
     return ans;
 }
 
