@@ -589,7 +589,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP x = CAR(args), y = CADR(args), xdims, ydims, ans;
     Rboolean sym;
 
-    if(PRIMVAL(op) == 0 &&
+    if(PRIMVAL(op) == 0 && /* %*% is primitive, the others are .Internal() */
        (IS_S4_OBJECT(x) || IS_S4_OBJECT(y))
        && R_has_methods(op)) {
 	SEXP s, value;
@@ -636,12 +636,18 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 		ncx = 1;
 	    }
 	}
-	else { /* crossprod */
+	else if (PRIMVAL(op) == 1) { /* crossprod() */
 	    if (LENGTH(x) == nry) {	/* x is a col vector */
 		nrx = nry; /* == LENGTH(x) */
 		ncx = 1;
 	    }
 	}
+	else { /* tcrossprod */
+	    if (LENGTH(x) == ncy) {	/* x as row vector */
+		nrx = 1;
+		ncx = ncy; /* == LENGTH(x) */
+	    }
+	}
     }
     else if (ldy != 2) {		/* y not a matrix */
 	nrx = INTEGER(xdims)[0];
@@ -658,7 +664,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 		ncy = LENGTH(y);
 	    }
 	}
-	else {
+	else { /* (t)crossprod */
 	    if (LENGTH(y) == nrx) {	/* y is a col vector */
 		nry = nrx; /* == LENGTH(y) */
 		ncy = 1;
