@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2008  Robert Gentleman, Ross Ihaka and the
+ *  Copyright (C) 1997--2009  Robert Gentleman, Ross Ihaka and the
  *                            R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -433,9 +433,9 @@ SEXP attribute_hidden do_strsplit(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef SUPPORT_MBCS
     if (fixed_opt || perl_opt) {
 	for (i = 0; i < tlen; i++)
-	if (getCharCE(STRING_ELT(tok, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	if (getCharCE(STRING_ELT(tok, i)) == CE_UTF8) use_UTF8 = TRUE;
 	for (i = 0; i < len; i++)
-	    if (getCharCE(STRING_ELT(x, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	    if (getCharCE(STRING_ELT(x, i)) == CE_UTF8) use_UTF8 = TRUE;
     }
     if (use_UTF8 && !fixed_opt && perl_opt) options = PCRE_UTF8;
 #endif
@@ -1071,7 +1071,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
     if ((fixed_opt || perl_opt) && !useBytes) {
 	if (getCharCE(STRING_ELT(pat, 0)) == CE_UTF8) use_UTF8 = TRUE;
 	for (i = 0; i < n; i++)
-	    if (getCharCE(STRING_ELT(vec, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	    if (getCharCE(STRING_ELT(vec, i)) == CE_UTF8) use_UTF8 = TRUE;
     }
     if (use_UTF8) {
 	cpat = translateCharUTF8(STRING_ELT(pat, 0));
@@ -1165,7 +1165,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (value_opt) {
 	SEXP nmold = getAttrib(vec, R_NamesSymbol), nm;
-	ans = allocVector(STRSXP, nmatches);
+	PROTECT(ans = allocVector(STRSXP, nmatches));
 	for (i = 0, j = 0; i < n ; i++)
 	    if (invert ^ LOGICAL(ind)[i])
 		SET_STRING_ELT(ans, j++, STRING_ELT(vec, i));
@@ -1177,6 +1177,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
 		    SET_STRING_ELT(nm, j++, STRING_ELT(nmold, i));
 	    setAttrib(ans, R_NamesSymbol, nm);
 	}
+	UNPROTECT(1);
     } else {
 	ans = allocVector(INTSXP, nmatches);
 	j = 0;
@@ -1339,7 +1340,7 @@ SEXP attribute_hidden do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
 	if (getCharCE(STRING_ELT(pat, 0)) == CE_UTF8) use_UTF8 = TRUE;
 	if (getCharCE(STRING_ELT(rep, 0)) == CE_UTF8) use_UTF8 = TRUE;
 	for (i = 0; i < n; i++)
-	    if (getCharCE(STRING_ELT(vec, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	    if (getCharCE(STRING_ELT(vec, i)) == CE_UTF8) use_UTF8 = TRUE;
     }
 
     if (use_UTF8) {
@@ -1411,14 +1412,13 @@ SEXP attribute_hidden do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
 		SET_STRING_ELT(ans, i, NA_STRING);
 	    else {
 		if (global) { /* need to find number of matches */
+		    const char *ss= s;
+		    int sst = st;
 		    nr = 0;
 		    do {
 			nr++;
-			s += st+patlen;
-		    } while((st = fgrep_one_bytes(spat, s, useBytes)) >= 0);
-		    /* and reset */
-		    s = translateChar(STRING_ELT(vec, i));
-		    st = fgrep_one_bytes(spat, s, useBytes);
+			ss += sst+patlen;
+		    } while((sst = fgrep_one_bytes(spat, ss, useBytes)) >= 0);
 		} else nr = 1;
 		cbuf = u = CallocCharBuf(ns + nr*(replen - patlen));
 		*u = '\0';
@@ -1463,7 +1463,10 @@ SEXP attribute_hidden do_gsub(SEXP call, SEXP op, SEXP args, SEXP env)
 	    else {
 		offset = 0;
 		nmatch = 0;
-		s = translateChar(STRING_ELT(vec, i));
+		if(use_UTF8)
+		    s = translateCharUTF8(STRING_ELT(vec, i));
+		else
+		    s = translateChar(STRING_ELT(vec, i));
 		t = srep;
 		cbuf = u = CallocCharBuf(ns);
 		ns = strlen(s);
@@ -1553,7 +1556,7 @@ SEXP attribute_hidden do_regexpr(SEXP call, SEXP op, SEXP args, SEXP env)
     if ((fixed_opt || perl_opt) && !useBytes) {
 	if (getCharCE(STRING_ELT(pat, 0)) == CE_UTF8) use_UTF8 = TRUE;
 	for (i = 0; i < n; i++)
-	    if (getCharCE(STRING_ELT(text, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	    if (getCharCE(STRING_ELT(text, i)) == CE_UTF8) use_UTF8 = TRUE;
     }
     if (use_UTF8) {
 	spat = translateCharUTF8(STRING_ELT(pat, 0));
@@ -1983,7 +1986,7 @@ SEXP attribute_hidden do_gregexpr(SEXP call, SEXP op, SEXP args, SEXP env)
     if (fixed_opt && !useBytes) {
 	if (getCharCE(STRING_ELT(pat, 0)) == CE_UTF8) use_UTF8 = TRUE;
 	for (i = 0; i < n; i++)
-	    if (getCharCE(STRING_ELT(text, 0)) == CE_UTF8) use_UTF8 = TRUE;
+	    if (getCharCE(STRING_ELT(text, i)) == CE_UTF8) use_UTF8 = TRUE;
     }
     if (use_UTF8) {
 	spat = translateCharUTF8(STRING_ELT(pat, 0));
