@@ -218,7 +218,6 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
     }
     PROTECT(svec = allocVector(STRSXP, localData.linenumber));
     deparse2(call, svec, &localData);
-    UNPROTECT(1);
     if (abbrev) {
 	char data[14];
 	strncpy(data, CHAR(STRING_ELT(svec, 0)), 10);
@@ -228,6 +227,9 @@ static SEXP deparse1WithCutoff(SEXP call, Rboolean abbrev, int cutoff,
     } else if(need_ellipses) {
 	SET_STRING_ELT(svec, R_BrowseLines, mkChar("  ..."));
     }
+    if(nlines > 0 && localData.linenumber < nlines)
+	svec = lengthgets(svec, localData.linenumber);
+    UNPROTECT(1); /* new version does not need to be protected */
     R_print.digits = savedigits;
     if ((opts & WARNINCOMPLETE) && localData.isS4)
 	warning(_("deparse of an S4 object will not be source()able"));
@@ -354,7 +356,7 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 	SET_TAG(o, install(translateChar(STRING_ELT(names, j))));
 	SETCAR(o, findVar(TAG(o), source));
 	if (CAR(o) == R_UnboundValue)
-	    warning(_("Object \"%s\" not found"), CHAR(PRINTNAME(TAG(o))));
+	    warning(_("object '%s' not found"), CHAR(PRINTNAME(TAG(o))));
 	else nout++;
     }
     o = objs;
@@ -365,9 +367,9 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 		if (CAR(o) == R_UnboundValue) continue;
 		obj_name = translateChar(STRING_ELT(names, i));
 		SET_STRING_ELT(outnames, nout++, STRING_ELT(names, i));
-		Rprintf(/* figure out if we need to quote the name */
-			isValidName(obj_name) ? "%s <-\n" : "`%s` <-\n",
-			obj_name);
+		if(isValidName(obj_name)) Rprintf("%s <-\n", obj_name);
+		else if(opts & S_COMPAT) Rprintf("\"%s\" <-\n", obj_name);
+		else Rprintf("`%s` <-\n", obj_name);
 		tval = deparse1(CAR(o), 0, opts);
 		for (j = 0; j < LENGTH(tval); j++)
 		    Rprintf("%s\n", CHAR(STRING_ELT(tval, j)));/* translated */
@@ -387,11 +389,18 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 		error(_("cannot write to this connection"));
 	    for (i = 0, nout = 0; i < nobjs; i++) {
 		const char *s;
+		int extra = 6;
 		if (CAR(o) == R_UnboundValue) continue;
 		SET_STRING_ELT(outnames, nout++, STRING_ELT(names, i));
 		s = translateChar(STRING_ELT(names, i));
-		res = Rconn_printf(con, "`%s` <-\n", s);
-		if(!havewarned && res < strlen(s) + 6)
+		if(isValidName(s)) {
+		    extra = 4;
+		    res = Rconn_printf(con, "%s <-\n", s);
+		} else if(opts & S_COMPAT)
+		    res = Rconn_printf(con, "\"%s\" <-\n", s);
+		else
+		    res = Rconn_printf(con, "`%s` <-\n", s);
+		if(!havewarned && res < strlen(s) + extra)
 		    warning(_("wrote too few characters"));
 		tval = deparse1(CAR(o), 0, opts);
 		for (j = 0; j < LENGTH(tval); j++) {
@@ -657,7 +666,15 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 	    attr1(s, d);
 	    print2buff("quote(", d);
 	}
-	if (d->backtick)
+	if (localOpts & S_COMPAT) {
+	    const char *ss = CHAR(PRINTNAME(s));
+	    if (isValidName(ss)) print2buff(ss, d);
+	    else {
+		print2buff("\"", d);
+		print2buff(ss, d);
+		print2buff("\"", d);
+	    }
+	} else if (d->backtick)
 	    print2buff(backquotify(CHAR(PRINTNAME(s))), d);
 	else
 	    print2buff(CHAR(PRINTNAME(s)), d);
@@ -883,7 +900,11 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		case PP_RETURN:
 		    if (isValidName(CHAR(PRINTNAME(op)))) /* ASCII */
 			print2buff(CHAR(PRINTNAME(op)), d);
-		    else {
+		    else if (d->backtick) {
+			print2buff("`", d);
+			print2buff(CHAR(PRINTNAME(op)), d);
+			print2buff("`", d);
+		    } else {
 			print2buff("\"", d);
 			print2buff(CHAR(PRINTNAME(op)), d);
 			print2buff("\"", d);
@@ -1002,9 +1023,15 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    print2buff("next", d);
 		    break;
 		case PP_SUBASS:
-		    print2buff("`", d);
-		    print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
-		    print2buff("`(", d);
+		    if(d->opts & S_COMPAT) {
+			print2buff("\"", d);
+			print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
+			print2buff("\'(", d);
+		    } else {
+			print2buff("`", d);
+			print2buff(CHAR(PRINTNAME(op)), d); /* ASCII */
+			print2buff("`(", d);
+		    }
 		    args2buff(s, 0, 0, d);
 		    print2buff(")", d);
 		    break;
@@ -1053,13 +1080,17 @@ static void deparse2buff(SEXP s, LocalParseData *d)
 		    else {
 			if ( isSymbol(CAR(s)) ){
 			    const char *ss = CHAR(PRINTNAME(CAR(s)));
-			    if ( !isValidName(ss) ){
-
+			    if (isValidName(ss))
+				print2buff(ss, d);
+			    else if(d->opts & S_COMPAT) {
+				print2buff("\'", d);
+				print2buff(ss, d);
+				print2buff("\'", d);
+			    } else {
 				print2buff("`", d);
 				print2buff(ss, d);
 				print2buff("`", d);
-			    } else
-				print2buff(ss, d);
+			    }
 			}
 			else
 			    deparse2buff(CAR(s), d);
@@ -1316,7 +1347,7 @@ static Rboolean src2buff(SEXP sv, int k, LocalParseData *d)
     SEXP t;
     int i, n;
 
-    if (length(sv) > k && !isNull(t = VECTOR_ELT(sv, k))) {
+    if (TYPEOF(sv) == VECSXP && length(sv) > k && !isNull(t = VECTOR_ELT(sv, k))) {
 	PROTECT(t);
 
 	PROTECT(t = lang2(install("as.character"), t));
@@ -1346,9 +1377,11 @@ static void vec2buff(SEXP v, LocalParseData *d)
     nv = getAttrib(v, R_NamesSymbol);
     if (length(nv) == 0) nv = R_NilValue;
 
-    if (d->opts & USESOURCE)
+    if (d->opts & USESOURCE) {
 	sv = getAttrib(v, R_SrcrefSymbol);
-    else
+	if (TYPEOF(sv) != VECSXP)
+	    sv = R_NilValue;
+    } else
 	sv = R_NilValue;
 
     for(i = 0 ; i < n ; i++) {
@@ -1360,7 +1393,11 @@ static void vec2buff(SEXP v, LocalParseData *d)
 	    /* d->opts = SIMPLEDEPARSE; This seems pointless */
 	    if( isValidName(translateChar(STRING_ELT(nv, i))) )
 		deparse2buff(STRING_ELT(nv, i), d);
-	    else {
+	    else if(d->backtick) {
+		print2buff("`", d);
+		deparse2buff(STRING_ELT(nv, i), d);
+		print2buff("`", d);
+	    } else {
 		print2buff("\"", d);
 		deparse2buff(STRING_ELT(nv, i), d);
 		print2buff("\"", d);
@@ -1388,7 +1425,11 @@ static void args2buff(SEXP arglist, int lineb, int formals, LocalParseData *d)
 	    const char *ss = CHAR(PRINTNAME(s));
 	    if( s == R_DotsSymbol || isValidName(ss) )
 		print2buff(ss, d);
-	    else {
+	    else if(d->backtick) {
+		print2buff("`", d);
+		print2buff(ss, d);
+		print2buff("`", d);
+	    } else {
 		print2buff("\"", d);
 		print2buff(ss, d);
 		print2buff("\"", d);
