@@ -127,10 +127,10 @@ static R_INLINE SEXP evalSym(SEXP e, SEXP rho)
         checkLocationConsistency(e, rho, loc, val);
     }
     if (val == R_UnboundValue)
-        error(_("object \"%s\" not found"), CHAR(PRINTNAME(e)));
+        error(_("object '%s' not found"), CHAR(PRINTNAME(e)));
     else if (val == R_MissingArg && !DDVAL(e)) {
         if (CHAR(PRINTNAME(e))[0])
-            error(_("argument \"%s\" is missing, with no default"),
+            error(_("argument '%s' is missing, with no default"),
                   CHAR(PRINTNAME(e)));
         else
             error(_("argument is missing, with no default"));
