@@ -1319,35 +1319,9 @@ static void findmethod(SEXP Class, const char *group, const char *generic,
     *which = whichclass;
 }
 
-/* a portion of the logic of R_data_class2, but not producing
-   "matrix", "array" classes or vector types as classes  */
-static SEXP data_class_group(SEXP obj) {
-    SEXP klass = getAttrib(obj, R_ClassSymbol);
-      if(length(klass) > 0) {
-	if(IS_S4_OBJECT(obj) && TYPEOF(obj) != S4SXP) { 
-	    /* try to return an S3Class slot, but NOT matrix/array */
-	    /* The S4 class is included for compatibility with
-	       the deprecated practice of defining S3 methods 
-	       for S4 classes.  Someday this should be disallowed. 
-	       JMC iii.9.09 */
-  	    SEXP s3class = S3Class(obj);
-	    if(s3class != R_NilValue) {
-	        SEXP value; int i, n = length(s3class);
-		PROTECT(value =  allocVector(STRSXP, n+1));
-		SET_STRING_ELT(value, 0, STRING_ELT(klass, 0));
-		for(i=0; i<n; i++)
-		  SET_STRING_ELT(value, i+1, STRING_ELT(s3class, i));
-		UNPROTECT(1);
-		return value;
-	    }
-	}
-      }
-      return(klass);
-}
-
 attribute_hidden
 int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
-                  SEXP *ans)
+		  SEXP *ans)
 {
     int i, j, nargs, lwhich, rwhich, set;
     SEXP lclass, s, t, m, lmeth, lsxp, lgr, newrho;
@@ -1410,11 +1384,11 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
 	error(_("generic name too long in '%s'"), PRIMNAME(op));
     sprintf(generic, "%s", PRIMNAME(op) );
 
-    lclass = IS_S4_OBJECT(CAR(args)) ? data_class_group(CAR(args))
+    lclass = IS_S4_OBJECT(CAR(args)) ? R_data_class2(CAR(args))
       : getAttrib(CAR(args), R_ClassSymbol);
 
     if( nargs == 2 )
-	rclass = IS_S4_OBJECT(CADR(args)) ? data_class_group(CADR(args))
+	rclass = IS_S4_OBJECT(CADR(args)) ? R_data_class2(CADR(args))
       : getAttrib(CADR(args), R_ClassSymbol);
     else
 	rclass = R_NilValue;
@@ -1425,21 +1399,15 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     findmethod(lclass, group, generic, &lsxp, &lgr, &lmeth, &lwhich,
 	       lbuf, rho);
     PROTECT(lgr);
-    if(isFunction(lsxp) && IS_S4_OBJECT(CAR(args))) {
-      if(lwhich > 0) {
+    if(isFunction(lsxp) && IS_S4_OBJECT(CAR(args)) && lwhich > 0
+       && isBasicClass(translateChar(STRING_ELT(lclass, lwhich)))) {
+	/* This and the similar test below implement the strategy
+	 for S3 methods selected for S4 objects.  See ?Methods */
         value = CAR(args);
 	if(NAMED(value)) SET_NAMED(value, 2);
-	value = asS4(value, 0, 2);
-	/* This and the similar test below are not possible when people
-	   insist on writing S3 methods for S4 classes & so NOT getting
-	   S4 inheritance JMC 8.iii.09 */
-/* 	if(TYPEOF(value) == S4SXP) */
-/* 	  error(_("Non-vector S4 object as first argument to operator")); */
-	SETCAR(args, value);
-      }
-      else { /* Design error: S3 method written for S4 class */
-	/*TODO: should warn here? */
-      }
+	value = R_getS4DataSlot(value, S4SXP); /* the .S3Class obj. or NULL*/
+	if(value != R_NilValue) /* use the S3Part as the inherited object */
+	  SETCAR(args, value);
     }
 
     if( nargs == 2 )
@@ -1448,18 +1416,13 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     else
 	rwhich = 0;
 
-    if(isFunction(rsxp) && IS_S4_OBJECT(CADR(args))) {
-      if(rwhich > 0) {
+    if(isFunction(rsxp) && IS_S4_OBJECT(CADR(args)) && rwhich > 0
+       && isBasicClass(translateChar(STRING_ELT(rclass, rwhich)))) {
         value = CADR(args);
 	if(NAMED(value)) SET_NAMED(value, 2);
-	value = asS4(value, 0, 2);
-/* 	if(TYPEOF(value) == S4SXP) */
-/* 	  error(_("Non-vector S4 object as second argument to binary operator")); */
-	SETCADR(args, value);
-      }
-      else {
-	/*TODO: should warn here ? */
-      }
+	value = R_getS4DataSlot(value, S4SXP);
+	if(value != R_NilValue)
+	  SETCADR(args, value);
     }
 
     PROTECT(rgr);
@@ -1493,7 +1456,8 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     PROTECT(m = allocVector(STRSXP,nargs));
     s = args;
     for (i = 0 ; i < nargs ; i++) {
-	t = getAttrib(CAR(s), R_ClassSymbol);
+	t = IS_S4_OBJECT(CAR(s)) ? R_data_class2(CAR(s))
+	  : getAttrib(CAR(s), R_ClassSymbol);
 	set = 0;
 	if (isString(t)) {
 	    for (j = 0 ; j < length(t) ; j++) {
@@ -1544,4 +1508,3 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     UNPROTECT(5);
     return 1;
 }
-
