@@ -805,6 +805,9 @@ SEXP bcEval(SEXP body, SEXP rho)
 	else error(_("invalid sequence argument in for loop"));
 	BCNPUSH(value);
 
+	/* bump up NAMED count of seq to avoid modification by loop code */
+	if (NAMED(seq) < 2) SET_NAMED(seq, NAMED(seq) + 1);
+
 	BCNPUSH(R_NilValue);
 
 	BC_CHECK_SIGINT();
