@@ -356,7 +356,14 @@ static SEXP for1(CSEXP call, CSEXP op, CSEXP args,
                  CSEXP sym, CSEXP rhs1, CSEXP body, CSEXP rho)
 {
     const int bgn = bodyHasBraces(body);
+
+    /* bump up NAMED count of sequence to avoid modification by loop code */
+    if (NAMED(rhs1) < 2) SET_NAMED(rhs1, NAMED(rhs1) + 1);
+
+    /***** nm may not be needed anymore now that NAMED(rhs1) is at
+	   least 1.  LT */
     const int nm = NAMED(rhs1);
+
     RCNTXT cntxt;
     PROTECT_INDEX api, vpi;
     volatile int i;             // index into rhs
