@@ -341,7 +341,7 @@ static R_INLINE void tracePopJitState(CSEXP call, Rboolean atTopLevel)
             printfSxp(call, "# Return from");
     }
 }
-static R_INLINE void traceMarkAsNoJittable(CSEXP s, const char msg[])
+static R_INLINE void traceMarkAsNotJittable(CSEXP s, const char msg[])
 {
     if (jitTrace >= 2 && !(s->sxpinfo.gp & CANNOT_JIT_BIT)) {
         if (TYPEOF(s) == LANGSXP)
@@ -455,20 +455,21 @@ static const char *getNameOfFuncThatCalledJit(void)
 
 static void nestedJitWarning(const unsigned jitArg, const unsigned traceArg)
 {
-    char thisFunc[STRLEN+1], originalFunc[STRLEN+1];
-
-    const char *str = getNameOfFuncThatCalledJit();
-    thisFunc[0] = 0;
-    if (str)
-        snprintf(thisFunc, STRLEN, _("in %s "), str);
-
+    char originalFunc[STRLEN+1];
     if (jitFuncName)
         snprintf(originalFunc, STRLEN, "%s ", jitFuncName);
     else
-        snprintf(originalFunc, STRLEN, _("a calling function "));
-
-    warning(_("ignored jit(%d) %sbecause %sis already jitting"),
-            jitArg, thisFunc, originalFunc);
+        snprintf(originalFunc, STRLEN, _("the code calling it "));
+    const char *thisFunc = getNameOfFuncThatCalledJit();
+    if (thisFunc)
+	warning(_("Ignored jit(%d) in %s because %sis jitted.\n"
+		  "  Functions called from jitted code cannot be jitted\n"
+                  "  and thus %s will not be jitted."),
+		jitArg, thisFunc, originalFunc, thisFunc);
+    else
+	warning(_("ignored jit(%d) because %sis already being jitted.\n"
+		  "  Functions called from jitted code cannot be jitted.\n"),
+		jitArg, originalFunc);
 }
 
 static int do_jitAux(SEXP call, int jitArg, int traceArg, int newTrace)
@@ -525,7 +526,7 @@ SEXP attribute_hidden do_jit(SEXP call, SEXP op, SEXP args, SEXP env)
     int jitArg = NA_INTEGER, traceArg = 0, newTrace = jitTrace;
     SEXP names = getAttrib(args, R_NamesSymbol);
     if (names != RNIL)
-        error("named arguments to \"jit\" are not supported");
+        error("named arguments to \"jit\" are not yet supported");
 
     checkArity(op, args);
 
@@ -560,7 +561,6 @@ SEXP attribute_hidden do_jit(SEXP call, SEXP op, SEXP args, SEXP env)
     extern int traceEvalFlag;
     traceEvalFlag = 1;
 #endif
-
     return getJitReturnVal();
 }
 
@@ -798,10 +798,10 @@ static R_INLINE void checkTypeAndLen(CSEXP sym, CSEXP oldval, CSEXP val)
 //
 // RA_TODO put in an explicit test somewhere to prevent user explicitly
 // using "sym <- NULL".  It's safe at the moment because jitUnresolved
-// catches it, but expeciit test would be better practice.
+// catches it, but explicit test would be better practice.
 //
-// The test against CANNOT_JIT_BIT is needed because the CANNOT_JIT_BIT 
-// takes precedence over the JITTED_BIT i.e. if both are set then the 
+// The test against CANNOT_JIT_BIT is needed because the CANNOT_JIT_BIT
+// takes precedence over the JITTED_BIT i.e. if both are set then the
 // sexp is not jitted.
 
 void checkJitBinding(CSEXP loc, CSEXP val)
@@ -993,14 +993,6 @@ void popJitState(CSEXP call)
 }
 
 
-void jitContextChangedAux(int nest)     // called by findcontext()
-{
-    // use "> 0" to skip nest==0 because applyClosure takes care of that
-
-    if (nest > 0)
-        popJitState(RNIL);
-}
-
 static int jitUnresolvedAtSuspend;
 
 // prevent genjit code generation for the moment
@@ -1019,6 +1011,23 @@ void jitUnsuspendAux(void) // reallow jit code generation
     setJitState(JITS_COMPILING, "jitUnsuspendAux");
 }
 
+// RA_TODO this is a hack (will go away when jit state nesting is fully implemented).
+// In newer versions of R, calling warning() from R leaves iStateStack 
+// one greater than its correct value.  So we call this function to force 
+// iStateStack to the correct value.
+// The proper fix will possibly require a change to the restart code.
+
+void saveJitStackPointer(Rboolean save)
+{
+    static int iOldStateStack;
+    if (save)
+        iOldStateStack = iStateStack;
+    else { // restore
+        assert(iStateStack == 0 || iStateStack == iOldStateStack + 1);
+        iStateStack = iOldStateStack;
+    }
+}
+
 //-----------------------------------------------------------------------------
 void decJitUnresolved(int n)
 {
@@ -1028,6 +1037,18 @@ void decJitUnresolved(int n)
     }
 }
 
+// RA_TODO to see why the named test below was removed, add following line of code
+//     if (s->sxpinfo.named) printfSxp(s, "isnamed\n");
+// and execute
+//    foo <- function() {
+//        jit(1, 3)
+//        ans <- matrix(0,nrow=3,ncol=2)
+//        # ans <- double(3) # will not be named if you include this
+//        x <- 7
+//        for(i in 1:3) ans[1,] <- 2 + x
+//    }
+//    foo()
+
 static R_INLINE void assertSxpInfoZero(SEXP s) // play safe
 {
     Dassert(TYPEOF(s) == LANGSXP);
@@ -1127,7 +1148,7 @@ static void terminateCompile(const char msg[])
 
 static R_INLINE void markAsNotJittable(CSEXP e, const char * const msg)
 {
-    traceMarkAsNoJittable(e, msg);
+    traceMarkAsNotJittable(e, msg);
     e->sxpinfo.gp |= CANNOT_JIT_BIT;
 }
 
@@ -1172,7 +1193,7 @@ static R_INLINE void fireUpCompiler(CSEXP e)
 
 // called by evalLang and genjitFor
 
-Rboolean jitProlog(CSEXP e, const char msg[])   
+Rboolean jitProlog(CSEXP e, const char msg[])
 {
     Rboolean fireup;
 
@@ -1419,7 +1440,7 @@ void genjitPushsym(SEXP loc, SEXP env)     // invoked for symbols
         disallowIfJitting(_("use a \"...\" variable"));
     Dassert(TYPEOF(loc) == LISTSXP || TYPEOF(loc) == SYMSXP);
     if (loc->sxpinfo.gp & CANNOT_JIT_BIT)
-        terminateCompile("user specified nojit");
+        terminateCompile("nojit bit is set");
     else {
         sym = getSymFromLoc(loc);
         Dassert(TYPEOF(sym) == SYMSXP);
@@ -1430,7 +1451,7 @@ void genjitPushsym(SEXP loc, SEXP env)     // invoked for symbols
             if (TYPEOF(val) == PROMSXP)
                 val = PRVALUE(val);
             Dassert(val != R_UnboundValue && val != RNIL);
-            switch(TYPEOF(val)) {
+            switch (TYPEOF(val)) {
                 case LGLSXP:
                 case INTSXP:
                 case REALSXP:
@@ -1457,7 +1478,7 @@ void genjitUnary(JIT_OPCODE opcode, CSEXP x) // invoked for unary minus etc.
 }
 
 static R_INLINE void binHelper(JIT_OPCODE opcode,
-                          SEXPTYPE ansType, int ansLen)
+                               SEXPTYPE ansType, int ansLen)
 {
     genjit(opcode, RNIL, NULL, NULL, 0, ansType, ansLen, RNIL, RNIL, RNIL);
 }
@@ -1707,17 +1728,17 @@ void genjitSubset2(CSEXP x, CSEXP subset) // for rhs expressions like x[i,j]
     };
     if (TYPEOF(x) == INTSXP || TYPEOF(x) == REALSXP) {
         assert(2 == length(getAttrib(x, R_DimSymbol)));
-	CSEXP index1 = CAR(subset);
-	CSEXP index2 = CADR(subset);
-	if ((TYPEOF(index1) == INTSXP || TYPEOF(index1) == REALSXP) &&
-		(TYPEOF(index2) == INTSXP || TYPEOF(index2) == REALSXP) &&
-		LENGTH(index2) == 1 && LENGTH(index2) == 1) {
-	    const int icode = 4 * (TYPEOF(x) == INTSXP) + 
-			      2 * (TYPEOF(index1) == INTSXP) + 
-			      (TYPEOF(index2) == INTSXP);
-	    JIT_OPCODE opcode = subsetOps[icode];
-	    genjit(opcode, RNIL, NULL, NULL, 0, NILSXP, 1, x, RNIL, RNIL);
-	}
+    CSEXP index1 = CAR(subset);
+    CSEXP index2 = CADR(subset);
+    if ((TYPEOF(index1) == INTSXP || TYPEOF(index1) == REALSXP) &&
+        (TYPEOF(index2) == INTSXP || TYPEOF(index2) == REALSXP) &&
+        LENGTH(index2) == 1 && LENGTH(index2) == 1) {
+        const int icode = 4 * (TYPEOF(x) == INTSXP) +
+                  2 * (TYPEOF(index1) == INTSXP) +
+                  (TYPEOF(index2) == INTSXP);
+        JIT_OPCODE opcode = subsetOps[icode];
+        genjit(opcode, RNIL, NULL, NULL, 0, NILSXP, 1, x, RNIL, RNIL);
+    }
     }
 }
 
@@ -1832,6 +1853,15 @@ void genjitSubas(CSEXP x, CSEXP index, CSEXP y)
 // The check and early return against jitUnresolved==0 are for
 // this (unusual) code: if (a) z <- if (b) 2   (notice no else).
 // But note that this code gets generated ok: if (a) z <- if (b) 2 else 3
+//
+// The check to see if the assignee has a dim attribute is because we don't
+// compile symbols with dimension attributes.  This is because evalJit does
+// not propagate attributes for efficienct (bearing mind that jitting 
+// expressions like z <- x + y where x and y are mats would not be much faster.
+// We call GetAttrib0 instead of GetAttr (which is slower) because the
+// attributes (if any) have already been set up.
+// TODO the difference in speed should be checked to see if this slightly
+// unstructured call to GetAttrib0 is worth it.
 
 void genjitAssign(CSEXP sym, SEXP loc, CSEXP y,  // y is the evaluated rhs
                   CSEXP env, const char msg[])
@@ -1856,31 +1886,36 @@ void genjitAssign(CSEXP sym, SEXP loc, CSEXP y,  // y is the evaluated rhs
         traceNoas();
         return;
     }
-    if (sym->sxpinfo.gp & CANNOT_JIT_BIT)
-        terminateCompile("user specified nojit");
-    else {
-        DCheckGenjitAssignParams(sym, loc, y, env, msg);
-        symval = getSymValFromLoc(loc);
-        if (symval == R_UnboundValue)
-            ;
-        else if (jitState == JITS_AWAITING_AS)
-            doSubas();      // subscripted assign: x[i] = y
-        else {              // standard assign: x = y
-            JIT_OPCODE opcode = adjustForTypes(JIT_as_r_r, symval, y);
-            if (ny == 1)
-                opcode += 3;                // r_r + 3 is r1_r1
-            assert(ny == LENGTH(symval));   // check that both side have same len
-            setJittedBit(loc);
+    if (sym->sxpinfo.gp & CANNOT_JIT_BIT) {
+        terminateCompile("nojit bit is set");
+        return;
+    }
+	DCheckGenjitAssignParams(sym, loc, y, env, msg);
+	symval = getSymValFromLoc(loc);
+	if (symval == R_UnboundValue)
+        return;
+	extern SEXP attribute_hidden getAttrib0(SEXP vec, SEXP name);
+	if (getAttrib0(symval, R_DimSymbol) != RNIL) {
+		terminateCompile("symbol has dim attribute");
+        return;
+	}
+	if (jitState == JITS_AWAITING_AS)
+		doSubas();      // subscripted assign: x[i] = y
+    else {              // standard assign: x = y
+		JIT_OPCODE opcode = adjustForTypes(JIT_as_r_r, symval, y);
+		if (ny == 1)
+			opcode += 3;                // r_r + 3 is r1_r1
+		assert(ny == LENGTH(symval));   // check that both side have same len
+		setJittedBit(loc);
 
-            // convert ny to size in bytes for memcpy in evalJitAs
-            if (ytype == LGLSXP || ytype == INTSXP)     
-                ny *= sizeof(int);
-            else // REALSXP
-                ny *= sizeof(double);
+		// convert ny to size in bytes for memcpy in evalJitAs
+		if (ytype == LGLSXP || ytype == INTSXP)
+			ny *= sizeof(int);
+		else // REALSXP
+			ny *= sizeof(double);
 
-            genjit(opcode, loc, NULL,  NULL, ny, 0, 0, y, sym, env);
-       }
-   }
+		genjit(opcode, loc, NULL,  NULL, ny, 0, 0, y, sym, env);
+	}
 }
 
 static void genjitEval(JIT_OPCODE opcode, CSEXP e, CSEXP env)
