@@ -207,7 +207,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
 		return(-1);
 	    state->bufp = state->buf;
     }
-#ifdef SHELL_ESCAPE
+#ifdef SHELL_ESCAPE /* not default */
     if (*state->bufp == '!') {
 #ifdef HAVE_SYSTEM
 	    R_system(&(state->buf[1]));
@@ -939,7 +939,6 @@ void setup_Rmainloop(void)
 	}
 	UNPROTECT(1);
     }
-    /* gc_inhibit_torture = 0; */
     {
 	int i;
 	for(i = 0 ; i < ndeferred_warnings; i++)
