@@ -18,8 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string */
-
 /* The x:y  primitive calls do_colon(); do_colon() calls cross_colon() if
    both arguments are factors and seq_colon() otherwise.
  */
@@ -209,6 +207,10 @@ static SEXP rep2(SEXP s, SEXP ncopy)
     default:
 	UNIMPLEMENTED_TYPE("rep2", s);
     }
+    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
+	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
+	SET_S4_OBJECT(a);
+    }
     if (inherits(s, "factor")) {
 	SEXP tmp;
 	if(inherits(s, "ordered")) {
@@ -227,6 +229,7 @@ static SEXP rep2(SEXP s, SEXP ncopy)
     return a;
 }
 
+/* called in do_rep_int() only */
 static SEXP rep1(SEXP s, SEXP ncopy)
 {
     int i, ns, na, nc;
@@ -255,6 +258,11 @@ static SEXP rep1(SEXP s, SEXP ncopy)
 	a = allocList(na);
     PROTECT(a);
 
+    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
+	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
+	SET_S4_OBJECT(a);
+    }
+
     switch (TYPEOF(s)) {
     case LGLSXP:
 	for (i = 0; i < na; i++)
@@ -406,6 +414,10 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 done:
     ans = do_subset_dflt(R_NilValue, R_NilValue, list2(x, ind), rho);
+    if(IS_S4_OBJECT(x)) { /* e.g. contains = "list" */
+	setAttrib(ans, R_ClassSymbol, getAttrib(x, R_ClassSymbol));
+	SET_S4_OBJECT(ans);
+    }
     /* 1D arrays get dimensions preserved */
     setAttrib(ans, R_DimSymbol, R_NilValue);
     UNPROTECT(nprotect);
@@ -479,7 +491,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
 
     if(lout == NA_INTEGER) {
-	double rfrom = asReal(from), rto = asReal(to), rby = asReal(by);
+	double rfrom = asReal(from), rto = asReal(to), rby = asReal(by), *ra;
 	if(from == R_MissingArg) rfrom = 1.0;
 	if(to == R_MissingArg) rto = 1.0;
 	if(by == R_MissingArg)
@@ -515,8 +527,13 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		errorcall(call, _("wrong sign in 'by' argument"));
 	    nn = (int)(n + FLT_EPSILON);
 	    ans = allocVector(REALSXP, nn+1);
+	    ra = REAL(ans);
 	    for(i = 0; i <= nn; i++)
-		REAL(ans)[i] = rfrom + i * rby;
+		ra[i] = rfrom + i * rby;
+	    /* Added in 2.9.0 */
+	    if (nn > 0)
+		if((rby > 0 && ra[nn] > rto) || (rby < 0 && ra[nn] < rto))
+		    ra[nn] = rto;
 	}
     } else if (lout == 0) {
 	ans = allocVector(INTSXP, 0);
@@ -586,7 +603,17 @@ SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
     int i, len, *p;
 
     checkArity(op, args);
+#ifdef R_291_and_less
     len = length(CAR(args));
+#else
+    if(isObject(CAR(args)) && DispatchOrEval(call, op, "length", args,
+					     rho, &ans, 0, 1)) {
+	len = asInteger(ans);
+    }
+    else
+	len = length(CAR(args));
+#endif
+
     ans = allocVector(INTSXP, len);
     p = INTEGER(ans);
     for(i = 0; i < len; i++) p[i] = i+1;
