@@ -17,9 +17,18 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
  *  http://www.r-project.org/Licenses/
- */
+ *
+ * EXPORTS:
+ *
+ *  OneIndex()        -- used for "[[<-" in ./subassign.c
+ *  get1index()       -- used for "[["   in ./subassign.c & subset.c
 
-/* <UTF8> char here is handled as a whole string */
+ *  mat2indsub()      -- for "mat[i]"     "    "            "
+
+ *  makeSubscript()   -- for "[" and "[<-" in ./subset.c and ./subassign.c
+ *  vectorSubscript() -- for makeSubscript()   {currently unused externally}
+ *  arraySubscript()  -- for "[i,j,..." and "[<-..." in ./subset.c, ./subassign.c
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -46,6 +55,7 @@ static int integerOneIndex(int i, int len, SEXP call)
     return(indx);
 }
 
+/* Utility used (only in) do_subassign2_dflt(), i.e. "[[<-" in ./subassign.c : */
 int attribute_hidden
 OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos, SEXP call)
 {
@@ -413,7 +423,6 @@ typedef SEXP (*StringEltGetter)(SEXP x, int i);
  * large, then it will be too slow unless ns is very small.
  */
 
-#define USE_HASHING 1
 static SEXP
 stringSubscript(SEXP s, int ns, int nx, SEXP names,
 		StringEltGetter strg, int *stretch, Rboolean in, SEXP call)
@@ -421,12 +430,8 @@ stringSubscript(SEXP s, int ns, int nx, SEXP names,
     SEXP indx, indexnames;
     int i, j, nnames, sub, extra;
     int canstretch = *stretch;
-#ifdef USE_HASHING
     /* product may overflow, so check factors as well. */
-    Rboolean usehashing = in && ( (ns > 1000 && nx) || (nx > 1000 && ns) || (ns * nx > 1000) );
-#else
-    Rboolean usehashing = FALSE;
-#endif
+    Rboolean usehashing = in && ( ((ns > 1000 && nx) || (nx > 1000 && ns)) && (ns * nx > 15*nx + ns) );
 
     PROTECT(s);
     PROTECT(names);
@@ -441,7 +446,6 @@ stringSubscript(SEXP s, int ns, int nx, SEXP names,
      * nonmatch will have given an error.)
      */
 
-#ifdef USE_HASHING
     if(usehashing) {
 	/* must be internal, so names contains a character vector */
 	/* NB: this does not behave in the same way with respect to ""
@@ -451,9 +455,9 @@ stringSubscript(SEXP s, int ns, int nx, SEXP names,
 	for (i = 0; i < ns; i++)
 	    if(STRING_ELT(s, i) == NA_STRING || !CHAR(STRING_ELT(s, i))[0])
 		INTEGER(indx)[i] = 0;
+	/* FIXME: this should not be allowed, CHARSXPs only */
 	for (i = 0; i < ns; i++) SET_STRING_ELT(indexnames, i, R_NilValue);
     } else {
-#endif
 	PROTECT(indx = allocVector(INTSXP, ns));
 	for (i = 0; i < ns; i++) {
 	    sub = 0;
@@ -465,6 +469,7 @@ stringSubscript(SEXP s, int ns, int nx, SEXP names,
 		    }
 		    if (NonNullStringMatch(STRING_ELT(s, i), names_j)) {
 			sub = j + 1;
+			/* FIXME: this should not be allowed, CHARSXPs only */
 			SET_STRING_ELT(indexnames, i, R_NilValue);
 			break;
 		    }
@@ -472,9 +477,8 @@ stringSubscript(SEXP s, int ns, int nx, SEXP names,
 	    }
 	    INTEGER(indx)[i] = sub;
 	}
-#ifdef USE_HASHING
     }
-#endif
+
 
     for (i = 0; i < ns; i++) {
 	sub = INTEGER(indx)[i];
