@@ -41,6 +41,10 @@
 #include "Defn.h"
 #include "jit.h"
 
+/* JMC convinced MM that this was not a good idea: */
+#undef _S4_subsettable
+
+
 /* ExtractSubset does the transfer of elements from "x" to "result" */
 /* according to the integer subscripts given in "indx". */
 
@@ -166,27 +170,36 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
 	SET_NAMED(result, 2);
 
     PROTECT(result = ExtractSubset(x, result, indx, call));
-    if (result != R_NilValue &&
-	(
+    if (result != R_NilValue) {
+	if (
 	    ((attrib = getAttrib(x, R_NamesSymbol)) != R_NilValue) ||
 	    ( /* here we might have an array.  Use row names if 1D */
 		isArray(x) && LENGTH(getAttrib(x, R_DimNamesSymbol)) == 1 &&
 		(attrib = getAttrib(x, R_DimNamesSymbol)) != R_NilValue &&
 		(attrib = GetRowNames(attrib)) != R_NilValue
 		)
-	    )) {
-	nattrib = allocVector(TYPEOF(attrib), n);
-	PROTECT(nattrib); /* seems unneeded */
-	nattrib = ExtractSubset(attrib, nattrib, indx, call);
-	setAttrib(result, R_NamesSymbol, nattrib);
-	UNPROTECT(1);
-    }
-    if (result != R_NilValue && (attrib = getAttrib(x, R_SrcrefSymbol)) != R_NilValue) {
-	nattrib = allocVector(VECSXP, n);
-	PROTECT(nattrib); /* seems unneeded */
-	nattrib = ExtractSubset(attrib, nattrib, indx, call);
-	setAttrib(result, R_SrcrefSymbol, nattrib);
-	UNPROTECT(1);
+	    ) {
+	    nattrib = allocVector(TYPEOF(attrib), n);
+	    PROTECT(nattrib); /* seems unneeded */
+	    nattrib = ExtractSubset(attrib, nattrib, indx, call);
+	    setAttrib(result, R_NamesSymbol, nattrib);
+	    UNPROTECT(1);
+	}
+	if ((attrib = getAttrib(x, R_SrcrefSymbol)) != R_NilValue &&
+	    TYPEOF(attrib) == VECSXP) {
+	    nattrib = allocVector(VECSXP, n);
+	    PROTECT(nattrib); /* seems unneeded */
+	    nattrib = ExtractSubset(attrib, nattrib, indx, call);
+	    setAttrib(result, R_SrcrefSymbol, nattrib);
+	    UNPROTECT(1);
+	}
+	/* FIXME:  this is wrong, because the slots are gone, so result is an invalid object of the S4 class! JMC 3/3/09 */
+#ifdef _S4_subsettable
+	if(IS_S4_OBJECT(x)) { /* e.g. contains = "list" */
+	    setAttrib(result, R_ClassSymbol, getAttrib(x, R_ClassSymbol));
+	    SET_S4_OBJECT(result);
+	}
+#endif
     }
     UNPROTECT(3);
     return result;
@@ -685,7 +698,8 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(nsubs < 2) {
 	SEXP dim = getAttrib(x, R_DimSymbol);
 	int ndim = length(dim);
-	ans = VectorSubset(ax, (nsubs == 1 ? CAR(subs) : R_MissingArg), call);
+	PROTECT(ans = VectorSubset(ax, (nsubs == 1 ? CAR(subs) : R_MissingArg),
+				   call));
 	/* one-dimensional arrays went through here, and they should
 	   have their dimensions dropped only if the result has
 	   length one and drop == TRUE
@@ -695,7 +709,6 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    int len = length(ans);
 
 	    if(!drop || len > 1) {
-		PROTECT(ans);
 		PROTECT(attr = allocVector(INTSXP, 1));
 		INTEGER(attr)[0] = length(ans);
 		setAttrib(ans, R_DimSymbol, attr);
@@ -709,7 +722,6 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    setAttrib(ans, R_NamesSymbol, R_NilValue);
 		    UNPROTECT(1);
 		}
-		UNPROTECT(1);
 	    }
 	}
     } else {
@@ -719,8 +731,8 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    ans = MatrixSubset(ax, subs, call, drop);
 	else
 	    ans = ArraySubset(ax, subs, call, drop);
+	PROTECT(ans);
     }
-    PROTECT(ans);
 
     /* Note: we do not coerce back to pair-based lists. */
     /* They are "defunct" in this version of R. */
@@ -740,9 +752,12 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     else {
 	PROTECT(ans);
     }
-    if (ATTRIB(ans) != R_NilValue) {
+    if (ATTRIB(ans) != R_NilValue) { /* remove probably erroneous attr's */
 	setAttrib(ans, R_TspSymbol, R_NilValue);
-	setAttrib(ans, R_ClassSymbol, R_NilValue);
+#ifdef _S4_subsettable
+	if(!IS_S4_OBJECT(x))
+#endif
+	    setAttrib(ans, R_ClassSymbol, R_NilValue);
     }
     UNPROTECT(4);
     return ans;
@@ -776,6 +791,7 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP ans, dims, dimnames, indx, subs, x;
     int i, ndims, nsubs, offset = 0;
     int drop = 1, pok, exact = -1;
+    int named_x;
 
     PROTECT(args);
     ExtractDropArg(args, &drop);
@@ -809,6 +825,13 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(nsubs > 1 && nsubs != ndims)
 	errorcall(call, _("incorrect number of subscripts"));
 
+    /* code to allow classes to extend environment */
+    if(TYPEOF(x) == S4SXP) {
+        x = R_getS4DataSlot(x, ANYSXP);
+	if(x == R_NilValue)
+	  errorcall(call, _("this S4 class is not subsettable"));
+    }
+
     /* split out ENVSXP for now */
     if( TYPEOF(x) == ENVSXP ) {
       if( nsubs != 1 || !isString(CAR(subs)) || length(CAR(subs)) != 1 )
@@ -832,6 +855,8 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (!(isVector(x) || isList(x) || isLanguage(x)))
 	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
 
+    named_x = NAMED(x);  /* x may change below; save this now.  See PR#13411 */
+
     if(nsubs == 1) { /* vector indexing */
 	SEXP thesub = CAR(subs);
 	int i = -1, len = length(thesub);
@@ -891,13 +916,13 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     if(isPairList(x)) {
 	ans = CAR(nthcdr(x, offset));
-	if (NAMED(x) > NAMED(ans))
-	    SET_NAMED(ans, NAMED(x));
+	if (named_x > NAMED(ans))
+	    SET_NAMED(ans, named_x);
     } else if(isVectorList(x)) {
 	/* did unconditional duplication before 2.4.0 */
 	ans = VECTOR_ELT(x, offset);
-	if (NAMED(x) > NAMED(ans))
-	    SET_NAMED(ans, NAMED(x));
+	if (named_x > NAMED(ans))
+	    SET_NAMED(ans, named_x);
     } else {
 	ans = allocVector(TYPEOF(x), 1);
 	switch (TYPEOF(x)) {
@@ -1016,6 +1041,12 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
 
     /* Optimisation to prevent repeated recalculation */
     slen = strlen(translateChar(input));
+     /* The mechanism to allow  a class extending "environment" */
+    if( IS_S4_OBJECT(x) && TYPEOF(x) == S4SXP ){
+        x = R_getS4DataSlot(x, ANYSXP);
+	if(x == R_NilValue)
+	    errorcall(call, "$ operator not defined for this S4 class");
+    }
 
     /* If this is not a list object we return NULL. */
     /* Or should this be allocVector(VECSXP, 0)? */
@@ -1128,9 +1159,6 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
     else if( isVectorAtomic(x) ){
 	errorcall(call, "$ operator is invalid for atomic vectors");
     }
-    else if( IS_S4_OBJECT(x) ){
-	errorcall(call, "$ operator not defined for this S4 class");
-    } 
     else /* e.g. a function */
 	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
     UNPROTECT(2);
