@@ -38,6 +38,9 @@
 
 #include <Defn.h>
 #include <stdlib.h>
+#include <Rinternals.h>
+#include <R_ext/Parse.h>  /* parsing is used in handling escape codes */
+
 
 #ifndef _Xconst
 #define _Xconst const
@@ -195,7 +198,7 @@ static char copycontents[sizeof(buf)+1] ;
 /* The next few and used only for the editor in MBCS locales */
 #ifdef USE_FONTSET
 static Status           status;
-static XFontSet         font_set;
+static XFontSet         font_set = NULL;
 static XFontStruct	**fs_list;
 static int		font_set_cnt;
 static char             fontset_name[]="-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*";
@@ -222,7 +225,7 @@ static XIMStyle status_styles[] = {
     XIMStatusNone,
     (XIMStyle)NULL,
 };
-static XIC ioic;
+static XIC ioic = NULL;
 #endif
 
 #ifndef max
@@ -397,6 +400,17 @@ SEXP in_RX11_dataentry(SEXP call, SEXP op, SEXP args, SEXP rho)
     endcontext(&cntxt);
     closewin(DE);
     if(nView == 0) {
+	if(fdView >= 0) { /* might be open after viewers, but unlikely */
+	    removeInputHandler(&R_InputHandlers,
+			       getInputHandler(R_InputHandlers,fdView));
+	    fdView = -1;
+	}
+#ifdef USE_FONTSET
+	if(font_set) {
+	    XFreeFontSet(iodisplay, font_set);
+	    font_set = NULL;
+	}
+#endif
 	XCloseDisplay(iodisplay);
 	iodisplay = NULL;
     }
@@ -1043,6 +1057,54 @@ static Rboolean getccol(DEstruct DE)
     return newcol;
 }
 
+static SEXP lang5(SEXP s, SEXP t, SEXP u, SEXP v, SEXP w)
+{
+    PROTECT(s);
+    s = LCONS(s, list4(t, u, v, w));
+    UNPROTECT(1);
+    return s;
+}
+
+static SEXP processEscapes(SEXP x)
+{
+    SEXP newval, pattern, replacement, expr;
+    ParseStatus status;
+    
+    /* We process escape sequences in a scalar string by escaping
+       unescaped quotes, then quoting the whole thing and parsing it.  This
+       is supposed to be equivalent to the R code
+
+       newval <- gsub(perl=TRUE, "(?<!\\\\)((\\\\\\\\)*)\"", "\\1\\\\\"", x)
+       newval <- sub('(^.*$)', '"\1"', newval)
+       newval <- eval(parse(text=newval))
+
+       We do it this way to avoid extracting the escape handling
+       code from the parser.  We need it in C code because this may be executed
+       numerous times from C in dataentry.c */
+    	
+    PROTECT( pattern = mkString("(?<!\\\\)((\\\\\\\\)*)\"") );
+    PROTECT( replacement = mkString("\\1\\\\\"") );
+    PROTECT( expr = lang5(install("gsub"), ScalarLogical(1), pattern, replacement, x) );
+    SET_TAG( CDR(expr), install("perl") );
+
+    PROTECT( newval = eval(expr, R_BaseEnv) );
+    PROTECT( pattern = mkString("(^.*$)") );
+    PROTECT( replacement = mkString("\"\\1\"") );
+    PROTECT( expr = lang4(install("sub"), pattern, replacement, newval) );
+    PROTECT( newval = eval(expr, R_BaseEnv) );
+    PROTECT( expr = R_ParseVector( newval, 1, &status, R_NilValue) );
+    
+    /* We only handle the first entry. If this were available more generally,
+       we'd probably want to loop over all of expr */
+       
+    if (status == PARSE_OK && length(expr))
+	PROTECT( newval = eval(VECTOR_ELT(expr, 0), R_BaseEnv) );
+    else
+	PROTECT( newval = R_NilValue );  /* protect just so the count doesn't change */
+    UNPROTECT(10);
+    return newval;
+}
+
 /* close up the entry to a cell, put the value that has been entered
    into the correct place and as the correct type */
 
@@ -1090,9 +1152,16 @@ static void closerect(DEstruct DE)
 		char *endp;
 		double new = R_strtod(buf, &endp);
 		Rboolean warn = !isBlankString(endp);
-		if (TYPEOF(cvec) == STRSXP)
-		    SET_STRING_ELT(cvec, wrow - 1, mkChar(buf));
-		else
+		if (TYPEOF(cvec) == STRSXP) {
+		    SEXP newval;
+		    PROTECT( newval = mkString(buf) );
+		    PROTECT( newval = processEscapes(newval) );
+		    if (TYPEOF(newval) == STRSXP && length(newval) == 1)
+			SET_STRING_ELT(cvec, wrow - 1, STRING_ELT(newval, 0));
+		    else
+			warning("dataentry: parse error on string");
+		    UNPROTECT(2);
+		} else
 		    REAL(cvec)[wrow - 1] = new;
 		if (newcol && warn) {
 		    /* change mode to character */
@@ -1490,11 +1559,16 @@ static void eventloop(DEstruct DE)
 {
     int done;
     DEEvent ioevent;
+    caddr_t temp;
 
     done = 0;
     while (done == 0) {
 	XNextEvent(iodisplay, &ioevent);
-	{
+	XFindContext(iodisplay, ioevent.xany.window, deContext, &temp);
+	if ((DEstruct) temp != DE) { /* so a View window */
+	    if (WhichEvent(ioevent) == Expose)
+		drawwindow((DEstruct) temp);
+	} else {
 #ifdef USE_FONTSET
 	    if (XFilterEvent(&ioevent, None)){
 		if(ioic){
@@ -1566,7 +1640,7 @@ static void R_ProcessX11Events(void *data)
     DEEvent ioevent;
     int done = 0;
 
-    while (XPending(iodisplay)) {
+    while (nView && XPending(iodisplay)) {
 	XNextEvent(iodisplay, &ioevent);
 	XFindContext(iodisplay, ioevent.xany.window, deContext, &temp);
 	DE = (DEstruct) temp;
@@ -1606,9 +1680,17 @@ static void R_ProcessX11Events(void *data)
 	free(DE);
 	nView--;
 	if(nView == 0) {
+	    /* NB: this is removing the handler that is currently
+	       being used: only OK to free here in R > 2.8.0 */
 	    removeInputHandler(&R_InputHandlers,
 			       getInputHandler(R_InputHandlers,fdView));
 	    fdView = -1;
+#ifdef USE_FONTSET
+	    if(font_set) {
+		XFreeFontSet(iodisplay, font_set);
+		font_set = NULL;
+	    }
+#endif
 	    XCloseDisplay(iodisplay);
 	    iodisplay = NULL;
 	}
@@ -1627,7 +1709,7 @@ static void doSpreadKey(DEstruct DE, int key, DEEvent * event)
     char *text;
 
     iokey = GetKey(event);
-    text = GetCharP(event);
+    if(DE->isEditor) text = GetCharP(event);
 
     if (CheckControl(event))
 	doControl(DE, event);
@@ -1656,16 +1738,21 @@ static void doSpreadKey(DEstruct DE, int key, DEEvent * event)
 #ifdef XK_Page_Down
     else if (iokey == XK_Page_Down) {
 	if(DE->isEditor)
-	   jumpwin(DE, DE->colmin, DE->rowmax);
+	    jumpwin(DE, DE->colmin, DE->rowmax);
 	else {
 	    int i = DE->ymaxused - DE->nhigh + 2;
-	    jumpwin(DE, DE->colmin, max(i, 1));
+            jumpwin(DE, DE->colmin, min(i, DE->rowmax));	    
 	}
 	cell_cursor_init(DE);
     }
 #elif defined(XK_Next)
     else if (iokey == XK_Next) {
-	jumpwin(DE, DE->colmin, DE->rowmax);
+	if(DE->isEditor)
+	    jumpwin(DE, DE->colmin, DE->rowmax);
+	else {
+	    int i = DE->ymaxused - DE->nhigh + 2;
+            jumpwin(DE, DE->colmin, min(i, DE->rowmax));	    
+	}
 	cell_cursor_init(DE);
     }
 #endif
@@ -1823,6 +1910,7 @@ void closewin(DEstruct DE)
 #endif
     XDestroyWindow(iodisplay, DE->iowindow);
     /* XCloseDisplay(iodisplay); */
+    Rsync(DE);
 }
 
 #define USE_Xt 1
@@ -1928,10 +2016,12 @@ static Rboolean initwin(DEstruct DE, const char *title) /* TRUE = Error */
 	    sprintf(opt_fontset_name, s, "medium", "r", 12);
 	} else strcpy(opt_fontset_name, fontset_name);
 
-	font_set = XCreateFontSet(iodisplay, opt_fontset_name,
-				  &missing_charset_list,
-				  &missing_charset_count, &def_string);
-	if (missing_charset_count) XFreeStringList(missing_charset_list);
+	if(font_set == NULL) {
+	    font_set = XCreateFontSet(iodisplay, opt_fontset_name,
+				      &missing_charset_list,
+				      &missing_charset_count, &def_string);
+	    if (missing_charset_count) XFreeStringList(missing_charset_list);
+	}
 	if (font_set == NULL) {
 	    warning("unable to create fontset %s", opt_fontset_name);
 	    return TRUE; /* ERROR */
@@ -2099,7 +2189,7 @@ static Rboolean initwin(DEstruct DE, const char *title) /* TRUE = Error */
     DE->iogc = XCreateGC(iodisplay, DE->iowindow, 0, 0);
 
 #ifdef USE_FONTSET
-    if(mbcslocale) {
+    if(mbcslocale && DE->isEditor) {
 	ioim = XOpenIM(iodisplay, NULL, NULL, NULL);
 	if(!ioim) {
 	    XDestroyWindow(iodisplay, DE->iowindow);
